""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import os
from helpers.logger import Logger
from helpers.cmd_utils import CmdUtils
from framework.base.tasks import Tasks
from constants import REPORT_FILE, STEP_RESULT_FILE, LOG_FILE

TASK_STATUS = {"DONE":"DONE", "FAILED":"FAILED"}
TASK_LOG_STATUS = {"STARTED":"STARTED","COMPLETED":"COMPLETED"}
TEXT_COLOR = {'GREEN':'\033[92m', 'RED':'\033[91m', 'YELLOW':'\033[93m', 'RESET':'\033[0m'}
TEXT_DECORATION = {'BLINK':'\033[5m', 'BOLD':'\033[1m','RESET':'\033[0m'}

class AddFortiSOARSupportUserOnFortiCloud(Tasks):
    TASK_STATUS_MSG = "Add FortiSOAR support user in case of FortiCloud instance"
    
    def __init__(self) -> None:
        super().__init__()
        self.logger = Logger.get_logger(__name__)
        self.cmd_line_utilities = CmdUtils()
        self.store_result_path = STEP_RESULT_FILE.format(self.current_version)
        self.report_path = REPORT_FILE.format(self.current_version)

    @property
    def tags(self) -> str:
        return 'post-upgrade'

    def get_description(self) -> str:
        return ""

    def is_supported(self) -> bool:
        current_version = int(self.current_version.replace('.', ''))
        target_upgrade_version = int(
            self.target_upgrade_version.replace('.', ''))
        return target_upgrade_version >= current_version

    def execute(self):
        self.add_banner_in_log_file(self.TASK_STATUS_MSG,TASK_LOG_STATUS["STARTED"])
        step_result = self.get_step_results('pre-upgrade', 'initialize')
        flag_is_forticloud_enterprise = step_result['flag_is_forticloud_enterprise']
        
        if flag_is_forticloud_enterprise:
            self._add_fortisoar_support_user_for_forticloud()
            
        self.add_banner_in_log_file(self.TASK_STATUS_MSG,TASK_LOG_STATUS["COMPLETED"])

    def validate(self) -> bool:
        return True

    def _print_status_msg(self, msg, status):
        reset = TEXT_COLOR["RESET"]
        if status == TASK_STATUS["DONE"]:
            color = TEXT_COLOR["GREEN"]
        else:
            color = TEXT_COLOR["RED"]
        truncated_message = msg[:65] + "..." if len(msg) > 65 else msg
        width = 8
        status = f"{status:^{width}}"
        colored_status = f"{color}{status}{reset}"
        final_msg = "{:<70}{}[{}]".format(truncated_message," ",colored_status)
        print(final_msg)
        
    def add_banner_in_log_file(self, msg:str, status: str) -> None:
        status_msg = " [{:^11}] {} {} ".format(status,":",msg)
        border_length = len(status_msg)
        border = '='*border_length
        new_line_char = "\n" if status==TASK_LOG_STATUS["STARTED"] else "\n\n"
        final_msg = f"{status_msg}{new_line_char}"
        if os.path.exists(LOG_FILE):
            with open(LOG_FILE,'a') as log_file:
                log_file.write(final_msg)
    
    def _add_fortisoar_support_user_for_forticloud(self):
        try:
            # Check and add Include statement for including sshd configuration files. 
            file_path = "/etc/ssh/sshd_config"
            search_line = "Include /etc/ssh/sshd_config.d/*.conf"

            with open(file_path, "r+") as file:
                lines = file.read()
                if search_line not in lines:
                    file.write(f"\n{search_line}\n")
                    print(f"'{search_line}' was not found and has been added to {file_path}.")
                    cmd_reload_sshd="systemctl reload sshd"
                    result = self.cmd_line_utilities.execute_cmd(cmd_reload_sshd)
                    print("SSHD service has been reloaded.")
                else:
                    print(f"'{search_line}' is already present in {file_path}.")

            # Add fortisoar_support user for forticloud instance during performing upgrade.
            cmd = "sh /opt/cyops/scripts/support_user_addition.sh"
            result = self.cmd_line_utilities.execute_cmd(cmd)
            if result['return_code'] != 0:
                self._print_status_msg(self.TASK_STATUS_MSG, TASK_STATUS["FAILED"])
                return
            self._print_status_msg(self.TASK_STATUS_MSG, TASK_STATUS["DONE"])
            
        except Exception as ex:
            err_msg = "ERROR: {}".format(ex)
            self.logger.exception(err_msg)
            self._print_status_msg(self.TASK_STATUS_MSG, TASK_STATUS["FAILED"])
            print(
                f"Exception occurred while adding fortisoar_support user for FortiCloud Instance. Refer logs at '{LOG_FILE}'"
            )